# Copyright (C) 2017  Lars Wirzenius
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import apifw


import ick2


class ControllerAPI:

    def __init__(self):
        self._state = ick2.ControllerState()

    def get_state_directory(self):
        return self._state.get_state_directory()

    def set_state_directory(self, dirname):
        self._state.set_state_directory(dirname)

    def load_projects(self):
        return self._state.load_projects()

    def find_missing_route(self, path):  # pragma: no cover
        return [
            {
                'method': 'GET',
                'path': '/version',
                'callback': self.get_version,
            },
            {
                'method': 'POST',
                'path': '/projects',
                'callback': self.post_callback(self.post_projects),
            },
            {
                'method': 'GET',
                'path': '/projects',
                'callback': self.get_callback(self.get_projects),
            },
            {
                'method': 'GET',
                'path': '/projects/<project>',
                'callback': self.get_callback(self.get_project),
            },
            {
                'method': 'PUT',
                'path': '/projects/<project>',
                'callback': self.put_callback(self.put_projects),
            },
            {
                'method': 'DELETE',
                'path': '/projects/<project>',
                'callback': self.get_callback(self.delete_projects),
            },
        ]

    def get_version(self, *args, **kwargs):
        return apifw.Response({
            'status': apifw.HTTP_OK,
            'body': {
                'version': ick2.__version__,
            },
            'headers': {
                'Content-Type': 'application/json',
            },
        })

    def get_callback(self, callback):  # pragma: no cover
        def wrapper(content_type, body, **kwargs):
            try:
                if 'raw_uri_path' in kwargs:
                    del kwargs['raw_uri_path']
                body = callback(**kwargs)
            except ick2.NotFound as e:
                return apifw.Response({
                    'status': apifw.HTTP_NOT_FOUND,
                    'body': str(e),
                    'headers': [],
                })
            return apifw.Response({
                'status': apifw.HTTP_OK,
                'body': body,
                'headers': {
                    'Content-Type': 'application/json',
                },
            })
        return wrapper

    def post_callback(self, callback):  # pragma: no cover
        def wrapper(content_type, body, **kwargs):
            body = callback(body)
            ick2.log.log('trace', msg_text='returned body', body=repr(body))
            return apifw.Response({
                'status': apifw.HTTP_CREATED,
                'body': body,
                'headers': {
                    'Content-Type': 'application/json',
                },
            })
        return wrapper

    def put_callback(self, callback):  # pragma: no cover
        def wrapper(content_type, body, **kwargs):
            if 'raw_uri_path' in kwargs:
                del kwargs['raw_uri_path']
            body = callback(body, **kwargs)
            ick2.log.log('trace', msg_text='returned body', body=repr(body))
            return apifw.Response({
                'status': apifw.HTTP_OK,
                'body': body,
                'headers': {
                    'Content-Type': 'application/json',
                },
            })
        return wrapper

    def get_projects(self):
        return {
            'projects': self._state.get_projects(),
        }

    def get_project(self, project=None):
        assert project is not None
        return self._state.get_project(project)

    def post_projects(self, project):
        return self._state.add_project(project)

    def put_projects(self, body, project=None):
        assert project is not None
        return self._state.update_project(project, body)

    def delete_projects(self, project):
        self._state.remove_project(project)
