# Copyright (C) 2017  Lars Wirzenius
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import tempfile
import unittest


import ick2


class ControllerAPITests(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.statedir = os.path.join(self.tempdir, 'state/dir')

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def create_api(self):
        api = ick2.ControllerAPI()
        api.set_state_directory(self.statedir)
        return api

    def test_has_no_state_directory_initially(self):
        api = ick2.ControllerAPI()
        statedir = api.get_state_directory()
        self.assertTrue(statedir is None)

    def test_sets_and_creates_state_directory(self):
        api = self.create_api()
        statedir = api.get_state_directory()
        self.assertEqual(statedir, self.statedir)
        self.assertTrue(os.path.exists(statedir))

    def test_returns_version_correcly(self):
        api = self.create_api()
        response = api.get_version()
        self.assertEqual(
            response['body'],
            {
                'version': ick2.__version__,
            }
        )

    def test_has_not_projects_initially(self):
        api = self.create_api()
        self.assertEqual(api.get_projects(), {'projects': []})

    def test_creates_project(self):
        project = {
            'project': 'foo',
            'shell_steps': ['build'],
        }
        api = self.create_api()
        self.assertEqual(api.post_projects(project), project)
        self.assertEqual(api.get_projects(), {'projects': [project]})

    def test_loads_projects_from_state_directory(self):
        project = {
            'project': 'foo',
            'shell_steps': ['build'],
        }
        api = self.create_api()
        api.post_projects(project)

        api2 = self.create_api()
        api2.load_projects()
        self.assertEqual(api2.get_projects(), {'projects': [project]})

    def test_gets_named_project(self):
        project = {
            'project': 'foo',
            'shell_steps': ['build'],
        }
        api = self.create_api()
        api.post_projects(project)
        self.assertEqual(api.get_project('foo'), project)

    def test_updates_named_project(self):
        project_v1 = {
            'project': 'foo',
            'shell_steps': ['build'],
        }
        project_v2 = dict(project_v1)
        project_v2['shell_steps'] = ['build it using magic']
        api = self.create_api()
        api.post_projects(project_v1)
        updated = api.put_projects(project_v2, project='foo')
        self.assertEqual(updated, project_v2)
        self.assertEqual(api.get_project('foo'), project_v2)

    def test_deletes_named_project(self):
        project = {
            'project': 'foo',
            'shell_steps': ['build'],
        }
        api = self.create_api()
        api.post_projects(project)
        api.delete_projects('foo')
        self.assertEqual(api.get_projects(), {'projects': []})
        with self.assertRaises(ick2.NotFound):
            api.get_project('foo')

    def test_raises_errro_deleting_missing_project(self):
        api = self.create_api()
        with self.assertRaises(ick2.NotFound):
            api.delete_projects('foo')
