#!/usr/bin/python3
# Copyright (C) 2017  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging
import logging.handlers
import os
import sys


import apifw
import slog
import yaml


import ick2


transactions = slog.Counter()


def counter():
    return 'HTTP transaction {}'.format(transactions.increment())


def dict_logger(log, stack_info=None):
    ick2.log.log(exc_info=stack_info, **log)


default_config = {
    'token-public-key': None,
    'token-audience': None,
    'token-issuer': None,
    'log': [],
    'statedir': None,
}


def load_config(filename, defconf):
    conf = yaml.safe_load(open(filename, 'r'))
    actual_config = dict(defconf)
    actual_config.update(conf)
    return actual_config


def main():
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler(sys.stderr)
    logger.addHandler(handler)
    logging.info('Starting ick controller main program')

    try:

        config_filename = os.environ.get('ICK_CONTROLLER_CONFIG')
        logging.info('config filename %r', config_filename)
        if not config_filename:
            logging.error('no ICK_CONTROLLER_CONFIG in environment')
            raise Exception('No ICK_CONTROLLER_CONFIG defined in environment')

        logging.info('reading config from %r', config_filename)
        config = load_config(config_filename, default_config)

        logging.info('config is %r', config)
        ick2.setup_logging(config)

        ick2.log.log('info', msg_text='Ick2 controller starts', config=config)

        api = ick2.ControllerAPI()
        ick2.log.log('info', msg_text='created ControllerAPI')

        api.set_state_directory(config['statedir'])
        ick2.log.log(
            'info', msg_text='called ControllerAPI.set_state_directory')

        api.load_projects()
        ick2.log.log('info', msg_text='called ControllerAPI.load_projects')

        application = apifw.create_bottle_application(
            api, counter, dict_logger, config)
        ick2.log.log('info', msg_text='called apifw.create_bottle_application')

        return application
    except SystemExit:
        pass
    except BaseException as e:
        logging.error(str(e))
        sys.exit(1)


app = main()

# If we are running this program directly with Python, and not via
# gunicorn, we can use the Bottle built-in debug server, which can
# make some things easier to debug.

if __name__ == '__main__':
    print('running in debug mode')
    app.run(host='127.0.0.1', port=12765)
